/////////////////////////////////////////////////////////////////////////
////                                                                 ////
//// USB Oscilloscope                                                ////
//// VERSION HISTORY                                                 ////
//// March 1st, 2008: Initial Release.                                ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////

#if !defined(__PCH__)
 #error USB CDC Library requires PIC18
#endif

#include <18F2455.h>
#fuses HSPLL,NOWDT,NOMCLR,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL3,CPUDIV1,VREGEN,NOPBADEN
#use delay(clock=48000000)

#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7)
#use fast_IO(B)
#use fast_IO(A)

/////////////////////////////////////////////////////////////////////////////
//
// If you are using a USB connection sense pin, define it here.  If you are
// not using connection sense, comment out this line.  Without connection
// sense you will not know if the device gets disconnected.
//       (connection sense should look like this:
//                             100k
//            VBUS-----+----/\/\/\/\/\----- (I/O PIN ON PIC)
//                     |
//                     +----/\/\/\/\/\-----GND
//                             100k
//        (where VBUS is pin1 of the USB connector)
//
/////////////////////////////////////////////////////////////////////////////
#define USB_CON_SENSE_PIN PIN_E3

#include <usb_cdc.h>
#include <stdlib.h>

/////////////////////////////////////////////////////////////////////////////
//
// Configure the  I/O
//
/////////////////////////////////////////////////////////////////////////////
#define Reg1_Pin       PIN_A0 //PORTAbits.RA0
#define Start_Pin      PIN_A1 //PORTAbits.RA1   
#define Reg0_Pin       PIN_A2 //PORTAbits.RA2   
#define WR_Pin         PIN_A3 //PORTAbits.RA3   
#define Adr_L_Pin      PIN_A4 //PORTAbits.RA4   
#define AEN_Pin        PIN_A5 //PORTAbits.RA5   

#define DevA0_Pin      PIN_C2 //PORTCbits.RC2
#define DevA1_Pin      PIN_C1 //PORTCbits.RC1
#define DevA2_Pin      PIN_C0 //PORTCbits.RC0

//device state
#define WaitCmd 0
#define ReadCmd 1
#define ExecCmd 2
#define ReadADC 3
#define SendDat 4
#define WaitSync 5

#define CmdLen   6
#define CmdSize CmdLen+1

#define _debug_mon
#define AdcWait	10

//Global variables
   int OscMode,
	   CmdCnt,
	   IntCnt,
	   SyncMode,
       SyncLevel,
	   SyncVal;
   long OscRamAdr,      //Current RAM read addres
       OscPktSize,      //Transmit block size
       OscTimeBase,     //clock devider value
       OscTimeShift,    //sample count load value
       OscDelay,        //delay from start of sampling to enable sync 
       OldTmr1,PktCount,
	   SyncAdr;

   char CmdLine[CmdSize];//comand line bufer
   char dat_buf[4];//RAM data bufer

// #ifdef _debug_mon
//   long MonAdr,MonCnt;
// #endif 
//
// Local function definition
//
void DeviceInit(void);
void WrByte(int data);
void WriteAdr(long addr);
void ScanSync(void);
//int ConvData(int data);
//void SetDiv(void);

#int_timer1
void Timer1_Overflow()
{
 IntCnt--;
 if (IntCnt==0){
   disable_interrupts(INT_TIMER1);
   if (SyncMode==1){
     output_low(AEN_Pin);
     output_low(Start_Pin);
	 OscRamAdr=get_timer1();
    }
  }  
}

void main(void) {
   char c,i;
   int AdcTimeOut;
   long arg;

   usb_init_cs();
   DeviceInit();
   enable_interrupts(GLOBAL);
   while (TRUE) {
     usb_task();
//
// TRANSMIT DATA FROM PERIPHERAL MODULE
//
     if (kbhit()) {
      c=getc();
      usb_cdc_putc(c);
     }
//
// Check data from host
//
     if (usb_cdc_kbhit()){
      c=usb_cdc_getc();
      putc(c); // send host data to peripheral module
      if (c > 0x1F){
       if (OscMode==WaitCmd)
        {OscMode=ReadCmd;CmdCnt=CmdLen;}
       if (OscMode==ReadCmd)
        {CmdLine[CmdLen-CmdCnt]=c;CmdCnt--;}
       if (CmdCnt==0)
        {OscMode=ExecCmd; }
      }else{
        if (c==0x1B){DeviceInit();}
        if (c==0x1A){OscMode=WaitCmd;}
        if (c==0x1F){
         usb_cdc_putc(0xAA);
         usb_cdc_putc(0x55);
         usb_cdc_putc(0xAA);
         usb_cdc_putc(0x00);
         usb_cdc_putc(OscMode);
        }
       }
     }
//
//Check & Exec actions by mode
//
     if (OscMode==ExecCmd){
     // Set time value 
      if (CmdLine[0]=='T'){
       CmdLine[0]='0';
     // Set Time Base
       if (CmdLine[1]=='B'){
        CmdLine[1]='x';
        OscTimeBase=atoi(CmdLine);
//      SetDiv();
        OscMode=WaitCmd;
       }
     // Set Time Shift  
       if (CmdLine[1]=='S'){
        CmdLine[1]='x';
        arg=atol(CmdLine);
        OscTimeShift=arg;
        OscMode=WaitCmd;
       }
      }     
     //Set BlockSize 
      if ((CmdLine[0]=='B')&&(CmdLine[1]=='S')){
         CmdLine[0]='0';
         CmdLine[1]='x';
         OscPktSize=atol(CmdLine);
         OscMode=WaitCmd;
      }
     //Set Adr Shift
      if ((CmdLine[0]=='A')&&(CmdLine[1]=='S')){
         CmdLine[0]='0';
         CmdLine[1]='x';
         OscDelay=atol(CmdLine);
         OscMode=WaitCmd;
      }
     //Set SyncLevel
      if ((CmdLine[0]=='S')&&(CmdLine[1]=='L')){
         CmdLine[0]='0';
         CmdLine[1]='x';
         SyncLevel=make8(atol(CmdLine),0);
         OscMode=WaitCmd;
      }
     //Start Capture
      if (CmdLine[0]=='S'&&CmdLine[1]=='t'&&CmdLine[2]=='a'){
       set_timer1(0);
	   AdcTimeOut=(OscDelay+OscPktSize)/8;
	   IntCnt=3;
       OldTmr1=0;
       output_high(AEN_Pin);
	   enable_interrupts(INT_TIMER1);
//load counter
       output_low(Reg0_Pin);
       output_low(Reg1_Pin);
       WrByte(make8(OscTimeShift,0));
       output_high(Reg1_Pin);
       WrByte(make8(OscTimeShift,1));
//set sync edge(rise or fall)
       output_low(Reg1_Pin);
       output_high(Reg0_pin);
       WrByte(CmdLine[5]);
	   SyncMode=0;
	   if (CmdLine[5]>'1'){SyncMode=1;}
//set devider
//      SetDiv();
       set_tris_b(0x00);
       output_b(make8(OscTimeBase,0));	//set sample rate
       output_high(Start_Pin);			//enable RAM addr counter
       delay_us(8);						//wait reload sample counter
       OscMode=WaitSync;
      }
 #ifdef _debug_mon     
      if ((CmdLine[0]=='M')&&(CmdLine[1]=='R')){
         PktCount=0;
         i=4;
         OscMode=SendDat;
      }
      if ((CmdLine[0]=='M')&&(CmdLine[1]=='D')){
         CmdLine[0]='0';
         CmdLine[1]='x';
         OscRamAdr=atol(CmdLine);
         PktCount=0;
         i=4;
         OscMode=SendDat;
      }
      if ((CmdLine[0]=='M')&&(CmdLine[1]=='S')){
         CmdLine[0]='0';
         CmdLine[1]='x';
         OscRamAdr=atol(CmdLine);
         WriteAdr(OscRamAdr);		
         OscMode=WaitCmd;
	  }
      if ((CmdLine[0]=='M')&&(CmdLine[1]=='B')){
		if (CmdLine[5]=='0')
		 {output_low(Reg0_Pin);output_low(Reg1_Pin);}
		if (CmdLine[5]=='1')
		 {output_low(Reg0_Pin);output_high(Reg1_Pin);}
		if (CmdLine[5]=='2')
		 {output_high(Reg0_Pin);output_low(Reg1_Pin);}
		if (CmdLine[5]=='3')
		 {output_high(Reg0_Pin);output_high(Reg1_Pin);}
        OscMode=WaitCmd;
      }
 #endif
	//Analog module command recive - wait next cmd
      if (OscMode==ExecCmd){OscMode=WaitCmd;}
     }
     if (OscMode==WaitSync){	//wait until fill RAM minimum set of data
      OscRamAdr=get_timer1(); 
      if (OscRamAdr > AdcTimeOut){
       output_high(Reg0_Pin);	//enable sync
       output_high(Reg1_Pin);
       AdcTimeOut=AdcWait;
       OscMode=ReadADC;
      }
      else OldTmr1=OscRamAdr; 
     }
     if (OscMode==ReadAdc){
      if (OldTmr1==get_timer1()){
       AdcTimeOut--;
       if (AdcTimeOut==0){
        OscMode=SendDat;
        set_tris_b(0xFF);
        PktCount=0;

         //Get Current RAM Addres
        OscRamAdr=get_timer1();
//        usb_cdc_putc(make8(OscRamAdr,1));
//        usb_cdc_putc(make8(OscRamAdr,0));
        OscRamAdr*=8;
        if (input_state(DevA2_Pin)==1){OscRamAdr -=4;}
        if (input_state(DevA1_Pin)==1){OscRamAdr +=2;}
//        if (input_state(DevA0_Pin)==1){OscRamAdr++;}
         //move pointer to begin of data
		output_low(AEN_Pin);
        output_low(Start_Pin);
        OscRamAdr-=0-OscTimeShift;
        OscRamAdr-=OscDelay;
        i=4;
         //Start packet transmittion
        usb_cdc_putc(0xAA);
        usb_cdc_putc(0x55);
        usb_cdc_putc(0xAA);
        usb_cdc_putc(0x01);
       }
      }
      else {
       AdcTimeOut=AdcWait;
       OldTmr1=get_timer1();
	   if ((SyncMode==1)&&(IntCnt==0))
		{
//Search sync event for auto-start
		 PktCount=OscRamAdr-2;
         SyncVal=255;
		 SyncAdr=OscRamAdr;
         while ((SyncVal!=0)&&(OscRamAdr!=PktCount)){
            WriteAdr(OscRamAdr);
         	output_low(Reg0_Pin);
         	output_low(Reg1_Pin);
		 	delay_cycles(2);
			ScanSync();
         	output_high(Reg0_Pin);
         	delay_cycles(2);
			ScanSync();
         	output_low(Reg0_Pin);
         	output_high(Reg1_Pin);
         	delay_cycles(2);
		 	ScanSync();
         	output_high(Reg0_Pin);
         	delay_cycles(2);
		 	ScanSync();
         	OscRamAdr+=2;
		 }
		 OscRamAdr=SyncAdr;
		 PktCount=0;
		 OscMode=SendDat;
		 i=4;
         usb_cdc_putc(0xAA);
         usb_cdc_putc(0x55);
         usb_cdc_putc(0xAA);
         usb_cdc_putc(0x01);
		}
      }
     }
     if (OscMode==SendDat){
      while ((usb_cdc_putready())&&(PktCount <= OscPktSize)){
        if (i>3){
        //Read 4 bytes of data
         WriteAdr(OscRamAdr);
         output_low(Reg0_Pin);
         output_low(Reg1_Pin);
		 delay_cycles(2);
         dat_buf[0]=input_b();//ConvData(input_b());
         output_high(Reg0_Pin);
         delay_cycles(2);
		 dat_buf[1]=input_b();//ConvData(input_b());
         output_low(Reg0_Pin);
         output_high(Reg1_Pin);
         delay_cycles(2);
		 dat_buf[2]=input_b();//ConvData(input_b());
         output_high(Reg0_Pin);
         delay_cycles(2);
		 dat_buf[3]=input_b();//ConvData(input_b());
         OscRamAdr+=2;
         i=0;
        }
        usb_cdc_putc(dat_buf[i]);
        PktCount++;   
        i++;
      }
      if (PktCount > OscPktSize){
       OscMode=WaitCmd;
      }
    }
   }// end main loop 
}//end main()
void ScanSync()
{
if (abs(input_b()-SyncLevel) < SyncVal){
	SyncVal=abs(input_b()-SyncLevel);SyncAdr=OscRamAdr;}
}
/*
int ConvData(int data){
 int tmpdat;

 tmpdat=0;
 if (bit_test(data,0)) bit_set(tmpdat,7);
 if (bit_test(data,1)) bit_set(tmpdat,6);
 if (bit_test(data,2)) bit_set(tmpdat,5);
 if (bit_test(data,3)) bit_set(tmpdat,4);
 if (bit_test(data,4)) bit_set(tmpdat,3);
 if (bit_test(data,5)) bit_set(tmpdat,2);
 if (bit_test(data,6)) bit_set(tmpdat,1);
 if (bit_test(data,7)) bit_set(tmpdat,0);
 return(tmpdat);
}*/
void WriteAdr(long addr){
  set_tris_b(0x00);
  output_b(make8(addr,0));
  delay_cycles(2);
  output_high(Adr_L_Pin);
  delay_cycles(2);
  output_low(Adr_L_Pin);
  delay_cycles(2);
  output_high(Reg0_Pin);
  output_low(Reg1_Pin);
  WrByte(make8(addr,1));
  output_low(AEN_Pin);
}

void DeviceInit(void){
// init PORT's
  setup_adc(ADC_OFF);
  setup_adc_ports(NO_ANALOGS);
  setup_comparator(NC_NC_NC_NC);
  set_tris_a(0);
  output_a(0x20);
  set_tris_b(0xFF);
//set PORT_C[0..2] to input
  set_tris_c(get_tris_c()|7);
// setup Timer1
  setup_timer_1(T1_EXTERNAL);
//init variables
//  SyncLevel=0x40;
  OscDelay=0;
  OscRamAdr=0;
  OscPktSize=1024;
  OscTimeShift=0xFE00;
  OscTimeBase=0;
  OscMode=WaitCmd;
  CmdLine[CmdLen]=0;
  output_low(Start_Pin);
  output_low(AEN_Pin);
//  SetDiv();
}
/*void SetDiv(void){
  output_high(Reg1_Pin);
  output_high(Reg0_pin);
  delay_cycles(5);
  WrByte(make8(OscTimeBase,0));
}*/
void WrByte(int data){
  set_tris_b(0x00);
  output_b(data);
  delay_cycles(3); 
  output_high(WR_Pin);
  delay_cycles(3);
  output_low(WR_Pin);
  set_tris_b(0xFF);        
}
